#ifndef __ANIM__
#define __ANIM__

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <matmath.h>
#include <list.h>

#include "param.h"

#ifdef call
#undef call
#endif

class anim;

typedef int (anim::*methodfn)(param**,param*,param*,param*,param*,param*,param*,param*,param*);

class method
{
protected:

	char		*dname;
	methodfn	fn;

public:

	method(char *n,methodfn f)
	{
		fn = f;
		dname = strdup(n);
	}
	~method()
	{
		free(dname);
	}

	char	*name()
	{
		return dname;
	}

	int	call(anim *caller,param **pr = 0,param *p0 = 0,param *p1 = 0,param *p2 = 0,param *p3 = 0,
			param *p4 = 0,param *p5 = 0,param *p6 = 0,param *p7 = 0)
	{
		typedef int (anim::*rt)(param**,param*,param*,param*,param*,param*,param*,param*,param*);
		typedef rt *prt;

		rt	r;
		r = *((prt)(&fn));
		(caller->*r)(pr,p0,p1,p2,p3,p4,p5,p6,p7);

		return 0;
	}
};

class script
{
protected:

	char	*d_name;
	char	*d_script;

public:

	script(char *n,char *t)
	{
		d_name = strdup(n);
		d_script = strdup(t);
	}
	~script()
	{
		if(d_name) free(d_name);
		if(d_script) free(d_script);
	}

	void	setScript(char *t)
	{
		if(d_script) free(d_script);
		d_script = strdup(t);
	}
	char	*scripttext()
	{
		return d_script;
	}
	char	*name()
	{
		return d_name;
	}
};

class anim
{
protected:

	list<method*>	methodlist;
	list<script*>	scriptlist;

public:
	anim(){};
	~anim(){};

	void	listMethods()
	{
		int	t;

		for(t = 0;t < methodlist.length();t++)
			printf("%i. method: %s\n",t + 1,methodlist[t]->name());
	}

	int	call(char *n,param **pr = 0,param *p0 = 0,param *p1 = 0,param *p2 = 0,param *p3 = 0,
			param *p4 = 0,param *p5 = 0,param *p6 = 0,param *p7 = 0)
	{
		int	t;

		for(t = 0;t < methodlist.length();t++)
			if(strcmp(methodlist[t]->name(),n) == 0)
			{
				int	result;

				result = methodlist[t]->call(this,pr,p0,p1,p2,p3,p4,p5,p6,p7);

				if(pr == 0 || *pr == 0) printf("no return value\n");

				return result;
			}
		printf("no such method %s\n",n);

		return -1;
	}

	int	callscript(char *fn,char *tmpdir);


protected:

	int	addMethod(char *n,methodfn f)
	{
		methodlist += new method(n,f);

		return 0;
	}
	int	delMethod(char *n)
	{
		int	t;

		for(t = 0;t < methodlist.length();t++)
			if(strcmp(methodlist[t]->name(),n) == 0)
			{
				return (methodlist.deleteCurrent() == 0);
			}
		return -1;
	}

private:

	int	makeParam(param **p,char *pstr)
	{
		if(*pstr == '<')
		{
			float	x,y,z;

			sscanf(pstr,"<%f,%f,%f>",&x,&y,&z);
			*p = new pmvector(x,y,z);
		}
		else if(isdigit(*pstr))
		{
			float	d;

			sscanf(pstr,"%f",&d);
			*p = new pmreal(d);
		}
		else
		{
			*p = new pmtext(pstr);
		}

		return 0;
	}
};


#endif

