#ifndef __PARAM__
#define __PARAM__

#include <stdlib.h>
#include <string.h>

#include <matmath.h>
#include <list.h>

class param
{
protected:

	int	dvalid,pmtype;

public:

	param()
	{
		dvalid = 0;
		pmtype = 0;
	}
	virtual ~param()
	{
	}

	int	type() { return pmtype;}

	int	valid()
	{
		return dvalid;
	}
	void	setValidity(int v = 1)
	{
		dvalid = v;
	}
};

class pmbool : public param
{
protected:

	int	dbool;

public:

	pmbool(int b = 0) :
		param()
	{
		dbool = b;
		pmtype = 1;
	}

	int	bool()
	{
		return dbool;
	}

	void	setBool(int b)
	{
		dbool = b;
		setValidity();
	}
};

class pmchar : public param
{
protected:

	char	dchar;

public:

	pmchar(char c = 0) :
		param()
	{
		dchar = c;
		pmtype = 2;
	}

	char	getChar()
	{
		return dchar;
	}

	void	setChar(char c)
	{
		dchar = c;
		setValidity();
	}
};

class pmint : public param
{
protected:

	int	dint;

public:

	pmint(int i = 0) :
		param()
	{
		dint = i;
		pmtype = 3;
	}

	int	getInt()
	{
		return dint;
	}

	void	setInt(int i)
	{
		dint = i;
		setValidity();
	}
};

class pmreal : public param
{
protected:

	double	dreal;

public:

	pmreal(double r = 0) :
		param()
	{
		dreal = r;
		pmtype = 4;
	}

	double	real()
	{
		return dreal;
	}

	void	setReal(double r)
	{
		dreal = r;
		setValidity();
	}
};

class pmvector : public param
{
protected:

	Vector3	dvector;

public:

	pmvector(Vector3 v = Vector3(0,0,0)) :
		param()
	{
		dvector = v;
		pmtype = 5;
	}
	pmvector(double x = 0,double y = 0,double z = 0) :
		param()
	{
		dvector = Vector3(x,y,z);
		pmtype = 5;
	}

	Vector3	vector()
	{
		return dvector;
	}

	void	setVector(Vector3 v)
	{
		dvector = v;
		setValidity();
	}
};

class pmlist : public param
{
};

class pmarray : public param
{
};


class pmtext : public param
{
protected:

	char	*dtext;

public:

	pmtext(char *c = 0) :
		param()
	{
		dtext = c;
		pmtype = 8;
	}

	char	*getText()
	{
		return dtext;
	}

	void	setText(char *c)
	{
		dtext = c;
		setValidity();
	}
};

class pmobject : public param
{
protected:

	char	*dname;

public:

	pmobject(char *c = 0) :
		param()
	{
		dname = c;
		pmtype = 8;
	}

	char	*getName()
	{
		return dname;
	}

	void	setName(char *c)
	{
		dname = c;
		setValidity();
	}
};


#endif

