#include <stdio.h>

#include "blob.h"
#include "view.h"


blob::blob(base *p,char *n,double tr,texture *t) :
	csgobj(p,n,t)
{
	numtype = NUM_BLOB;
	threshold = tr;

	bloblist.setErrorValue(0);

	if(parent) addToParent(parent);
}

blob::blob(base *p,char *n,double tr,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	csgobj(p,n,vs,vt,vr)
{
	numtype = NUM_BLOB;
	threshold = tr;

	bloblist.setErrorValue(0);

	if(parent) addToParent(parent);
}

blob::blob(base *p,char *n,double tr,texture *t,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	csgobj(p,n,t,vs,vt,vr)
{
	numtype = NUM_BLOB;
	threshold = tr;

	bloblist.setErrorValue(0);

	if(parent) addToParent(parent);
}

blob::blob(base *p,blob *bc) :
	csgobj(p,(csgobj*)bc)
{
	int		t;

	numtype = NUM_BLOB;
	threshold = bc->threshold;

	bloblist.setErrorValue(0);
	for(t = 0;t < bc->bloblist.length();t++)
	{
		bc->bloblist[t]->copy(this);
	}

	if(parent) addToParent(parent);
}

blob::~blob()
{
	blobobj	*b;

	while((b = bloblist.getFirst()))
	{
		delete b;
		bloblist.deleteCurrent();
	}
	removeFromParent();
}

base	*blob::copy(base *p)
{
	return new blob(p,this);
}

int	blob::addToParent(base *p)
{
	if(!p) return -2;

	parent = p;
	return p->addChild(this);
}

int	blob::removeFromParent()
{
	if(!parent) return -2;
	return parent->removeChild(this);
}

int	blob::exportPOV(FILE *fp,int tab,int tabsize)
{
	blobobj	*b;

	printTab(fp,tab);
	fprintf(fp,"// Objectname = %s\n",name);
	printTab(fp,tab);
	fprintf(fp,"// Objecttype = blob\n");

	printTab(fp,tab);
	fprintf(fp,"blob\n");
	printTab(fp,tab);
	fprintf(fp,"{\n");
	printTab(fp,tab + tabsize);
	fprintf(fp,"threshold %f\n",threshold);

	for(b = bloblist.getFirst();b != 0;b = ++bloblist)
	{
		b->exportPOV(fp,tab + tabsize,tabsize);
		bloblist.find(b);
	}

	if(texptr) texptr->exportPOV(fp,tab + tabsize,tabsize);
	else texture("white",1,1,1).exportPOV(fp,tab + tabsize,tabsize);

	dim::exportPOV(fp,tab + tabsize,tabsize);

	printTab(fp,tab);
	fprintf(fp,"}\n");

	return 0;
}

void	blob::dumpNames(int tab,int tabsize)
{
	int		t;

	printTab(stdout,tab);
	printf("blob: %s\n",name);
	for(t = 0;t < bloblist.length();t++)
		bloblist[t]->dumpNames(tab + tabsize,tabsize);
}

int	blob::addChild(blobobj *b)
{
	if(!b) return -2;

	return bloblist += b;
}

int	blob::removeChild(blobobj *b)
{
	if(!b) return -2;
	if(bloblist.find(b) < 0) return -3;
	return (bloblist.deleteCurrent() ? 0 : -1);
}

int	blob::save(FILE *fp)
{
	blobobj	*b;

	if(!fp) return -1;

	setFile(fp);
	writeChunk("BLOB");
	SN(name);
	SD(threshold);

	for(b = bloblist.getFirst();b != 0;b = ++bloblist)
	{
		b->save(fp);
		bloblist.find(b);
	}

	dim::save(fp);

	saveTexture(fp);

	writeChunkLen();

	return 0;
}

int	blob::load(FILE *fp,int l)
{
	int	pos = ftell(fp);
	base	*b;

	fread(&threshold,1,sizeof(double),fp);

	while(ftell(fp) < pos + l - (int)sizeof(double) * 12)
	{
		b = parse(fp);
		if(b)
		{
			b->addToParent(this);
			b->setParent(this);
		}
	}

	dim::load(fp,l - (ftell(fp) - pos));

	if(l - (ftell(fp) - pos) > 0)
	{
		loadTexture(fp,l - (ftell(fp) - pos));
	}

	return 0;
}

base	*blob::searchName(char *n) 
{ 
	blobobj	*d; 
	char	*c; 
 	base	*b;

	if(n == 0) return 0; 
 
	c = getName(); 
	if(c != 0 && strcmp(c,n) == 0) 
		return this; 
 
	for(d = bloblist.getFirst();d != 0;d = ++bloblist) 
	{ 
		if((b = d->searchName(n))) return b; 
		bloblist.find(d);
	} 
 
	return 0; 
} 
 
int	blob::existsName(char *n) 
{ 
	blobobj	*d; 
	char	*c; 
 
	if(n == 0) return -1; 
 
	c = getName(); 
	if(c != 0 && strcmp(c,n) == 0) 
		return !0; 
 
	for(d = bloblist.getFirst();d != 0;d = ++bloblist) 
	{ 
		if(d->existsName(n)) return !0; 
		bloblist.find(d);
	} 
 
	return 0; 
} 

int	blob::draw(view *v,Matrix44 m)
{
	blobobj	*d;
	int	t;
	Vector3	vec(0,0,0);

	if(v == 0) return -1;

	dimMatrix(m);

	if(this == v->getSelected()) v->setDrawSelected(1);

	v->drawCross(vec,m);

	if(isFlag(DRAW_BOUNDINGBOX))
		drawBB(v,m);
	else
		for(t = 0;t < bloblist.length();t++)
		{
			d = bloblist[t];
			d->draw(v,m); 
		} 

	if(this == v->getSelected()) v->setDrawSelected(0);

	return 0;
} 

int	blob::calculate()
{
	blobobj		*d;
	int		t;
	Vector4		smin,smax;
	Matrix44	m;

	vmin = Vector3(1e100,1e100,1e100);
	vmax = Vector3(-1e100,-1e100,-1e100);

	for(t = 0;t < bloblist.length();t++)
	{
		d = bloblist[t];
		m.unify();
		d->dimMatrix(m);
		d->calculate();
		smin = Vector4(d->min(),1);
		smax = Vector4(d->max(),1);
		smin *= m;
		smax *= m;
		vmin[0] = MIN(vmin[0],smin[0]); 
		vmin[1] = MIN(vmin[1],smin[1]); 
		vmin[2] = MIN(vmin[2],smin[2]); 
		vmax[0] = MAX(vmax[0],smax[0]); 
		vmax[1] = MAX(vmax[1],smax[1]); 
		vmax[2] = MAX(vmax[2],smax[2]); 
	}

	return 0;
}



