#include <stdio.h>
#include <stdlib.h>

#include "refptr.h"

refptr::refptr(base *p,char *n,texture *t) :
	atom(p,n,t)
{
	numtype = NUM_REFPTR;
	refp = 0;
	tmpname = 0;

	if(parent)
		addToParent(parent);
}

refptr::refptr(base *p,char *n,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	atom(p,n,vs,vr,vt)
{
	numtype = NUM_REFPTR;
	refp = 0;
	tmpname = 0;

	if(parent)
		addToParent(parent);
}

refptr::refptr(base *p,char *n,texture *t,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	atom(p,n,t,vs,vr,vt)
{
	numtype = NUM_REFPTR;
	refp = 0;
	tmpname = 0;

	if(parent)
		addToParent(parent);
}

refptr::refptr(base *p,refptr *rc) :
	atom(p,(atom*)rc)
{
	numtype = NUM_REFPTR;
	refp = rc->refp;
	tmpname = rc->tmpname;

	if(parent)
		addToParent(parent);
}

refptr::~refptr()
{
	removeFromParent();
}

base	*refptr::copy(base *p)
{
	return new refptr(p,this);
}

int	refptr::addToParent(base *b)
{
	if(!b) return -2;

	parent = b;
	return b->addChild(this);
}

int	refptr::removeFromParent()
{
	if(!parent) return -2;

	return parent->removeChild(this);
}

int	refptr::addChild(ref *r)
{
	if(!r) return -2;

	refp = r;

	return 0;
}

void	refptr::getChildren(base ***b)
{
	base	**bt;

	*b = NULL;
	bt = (base**)malloc(sizeof(void*) * 2);
	if(!bt) return;

	bt[0] = 0; //refp;
	bt[1] = 0;
	*b = bt;
}

int	refptr::exportPOV(FILE *fp,int tab,int tabsize)
{
	if(!refp)
	{
		if(tmpname)
		{
			refp = (ref*)(getRoot()->searchName(tmpname));
		}

		if(refp)
		{
			if(tmpname) free(tmpname);
			tmpname = 0;
		}
	}
	if(!refp && !tmpname) return -1;

	printTab(fp,tab);
	fprintf(fp,"// Objectname = %s\n",name);
	printTab(fp,tab);
	fprintf(fp,"// Objecttype = refptr\n");

	printTab(fp,tab);
	fprintf(fp,"object\n");
	printTab(fp,tab);
	fprintf(fp,"{\n");
	printTab(fp,tab + tabsize);
	if(refp)
		fprintf(fp,"%s\n",refp->getName());
	else if(tmpname)
		fprintf(fp,"%s\n",tmpname);

	if(texptr) texptr->exportPOV(fp,tab + tabsize,tabsize);

	dim::exportPOV(fp,tab + tabsize,tabsize);

	printTab(fp,tab);
	fprintf(fp,"}\n");

	return 0;
}

void	refptr::dumpNames(int tab,int tabsize)
{
	printTab(stdout,tab);
	printf("refptr: %s\n",name);

	if(!refp)
	{
		if(tmpname)
		{
			refp = (ref*)(getRoot()->searchName(tmpname));
		}

		if(!refp) return;
		if(tmpname) free(tmpname);
		tmpname = 0;
	}
	refp->dumpNames(tab + tabsize,tabsize);
}

int	refptr::save(FILE *fp)
{
	if(!refp)
	{
		if(tmpname)
		{
			refp = (ref*)(getRoot()->searchName(tmpname));
		}

		if(!refp) return -1;
		if(tmpname) free(tmpname);
		tmpname = 0;
	}

	setFile(fp);
	writeChunk("REFP");
	SN(name);
	SN(refp->getName());
	dim::save(fp);
	saveTexture(fp);
	writeChunkLen();

	return 0;
}

int	refptr::load(FILE *fp,int l)
{
	int	len = 0;

	tmpname = readName(fp);
	printf("RefPTR: %s\n",tmpname);
/*
	if(tmpname)
	{
		root = getRoot();
		refp = (ref*)(root->searchName(tmpname));
		len = strlen(tmpname);
		if(refp && tmpname) free(tmpname);
	}
*/
	dim::load(fp,l - len - 4);
	if(l - len - 4 > (int)(9 * sizeof(double)))
	{
		loadTexture(fp,l - len - 4 - 9 * sizeof(double));
	}

	return 0;
}

base	*refptr::getPTR()
{
	if(!refp && tmpname)
	{
		base	*root;

		root = getRoot();
		refp = (ref*)(root->searchName(tmpname));
		if(refp && tmpname) free(tmpname);
	}
		
	return refp;
}

char	*refptr::getPTRName()
{
	if(refp) return refp->getName();
	if(tmpname) return tmpname;

	return 0;
}

int	refptr::draw(view *v,Matrix44 m)
{
	if(isFlag(HIDE)) return 0;

	dimMatrix(m);

	if(!refp)
	{
		if(tmpname)
		{
			refp = (ref*)(getRoot()->searchName(tmpname));
		}

		if(!refp) return -2;
		if(tmpname) free(tmpname);
		tmpname = 0;
	}
	return refp->drawRef(v,m);
}

int	refptr::calculate()
{
	Vector4		smin,smax;

	vmin = Vector3(1e100,1e100,1e100);
	vmax = Vector3(-1e100,-1e100,-1e100);

	refp->calculate();
	smin = Vector4(refp->min(),1);
	smax = Vector4(refp->max(),1);
	vmin[0] = MIN(vmin[0],smin[0]); 
	vmin[1] = MIN(vmin[1],smin[1]); 
	vmin[2] = MIN(vmin[2],smin[2]); 
	vmax[0] = MAX(vmax[0],smax[0]); 
	vmax[1] = MAX(vmax[1],smax[1]); 
	vmax[2] = MAX(vmax[2],smax[2]); 

	return 0;
}


