#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "text.h"
#include "num.h"
#include "view.h"


text::text(base *p,char *n,char *txt,char *fnt,double th,texture *t) :
	csgobj(p,n,t)
{
	numtype = NUM_TEXT;
	fontname = strdup(fnt);
	context = strdup(txt);
	thickness = th;
	offset = Vector2(0,0);

	fontptr = new font(fnt,fnt);
	fontcache = 0;

	if(parent) addToParent(parent);
}

text::text(base *p,char *n,char *txt,char *fnt,double th,Vector2 &offs,texture *t,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	csgobj(p,n,t,vs,vr,vt)
{
	numtype = NUM_TEXT;
	fontname = strdup(fnt);
	context = strdup(txt);
	thickness = th;
	offset = offs;

	fontptr = new font(fnt,fnt);
	fontcache = 0;

	if(parent) addToParent(parent);
}

text::text(base *p,char *n,char *txt,char *fnt,double th,Vector2 &offs,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	csgobj(p,n,vs,vr,vt)
{
	numtype = NUM_TEXT;
	fontname = strdup(fnt);
	context = strdup(txt);
	thickness = th;
	offset = offs;

	fontptr = new font(fnt,fnt);
	fontcache = 0;

	if(parent) addToParent(parent);
}

text::text(base *p,text *tc) :
	csgobj(p,(csgobj*)tc)
{
	numtype = NUM_TEXT;
	fontname = strdup(tc->fontname);
	context = strdup(tc->context);
	thickness = tc->thickness;
	offset = tc->offset;

	fontptr = tc->fontptr; // kopie erstellen
	fontcache = 0;

	if(parent) addToParent(parent);
}

text::~text()
{
	if(fontname)
		free(fontname);
	if(context)
		free(context);

	if(fontcache) free(fontcache);
	delete fontptr;
}

base	*text::copy(base *p)
{
	return new text(p,this);
}

int	text::addToParent(base *p)
{
	if(!p) return -2;

	parent = p;
	return p->addChild(this);
}

int	text::removeFromParent()
{
	if(!parent) return -2;

	return parent->removeChild(this);
}

void	text::dumpNames(int tab,int)
{
	printTab(stdout,tab);
	printf("text: %s\n",name);
}

int	text::exportPOV(FILE *fp,int tab,int tabsize) 
{ 
	printTab(fp,tab); 
	fprintf(fp,"// Objectname = %s\n",name); 
	printTab(fp,tab);
	fprintf(fp,"// Objecttype = text\n");
	printTab(fp,tab);
	fprintf(fp,"text\n");
	printTab(fp,tab); 
	fprintf(fp,"{\n"); 
	printTab(fp,tab + tabsize); 
	fprintf(fp,"ttf \"%s\", \n",fontname); 
	printTab(fp,tab + tabsize);
	fprintf(fp,"\"%s\",\n", context);
	printTab(fp,tab + tabsize);
	fprintf(fp,"%g <%g,%g>\n",thickness, offset(0),offset(1));
	
	if(texptr) texptr->exportPOV(fp,tab + tabsize,tabsize);

	dim::exportPOV(fp,tab + tabsize,tabsize);

	printTab(fp,tab);
	fprintf(fp,"}\n");

	return 0;
}

int	text::save(FILE *fp)
{
	if(!fp) return -1;

	setFile(fp);
	writeChunk("TEXT");
	SN(name);
	SN(fontname);
	SN(context);
	SD(thickness);
	SD(offset(0));
	SD(offset(1));

	dim::save(fp);

	writeChunkLen();

	return 0;
}

int	text::load(FILE *fp,int l)
{
	double	x,y;

	fontname = readName(fp);
	context = readName(fp);
	fread(&thickness,1,sizeof(double),fp);
	fread(&x,1,sizeof(double),fp);
	fread(&y,1,sizeof(double),fp);
	offset = Vector2(x,y);

	dim::load(fp,l);

	if(fontptr) fontptr->setFileName(fontname);

	return 0;
}

int	text::draw(view *v,Matrix44 m)
{
	char		*cp;
	fontglyph	*gp = 0;
	int		cc,sp,ep,pn,poc;
	Vector2		p[2048];
	double		x,y;

//	printf("Text: %s\n",context);

	if(isFlag(HIDE)) return 0;

	dimMatrix(m);

	if(this == v->getSelected()) v->setDrawSelected(1);

	if(isFlag(DRAW_BOUNDINGBOX))
		drawBB(v,m);
	else
	{
		x = 0;
		y = 0;

		for(cp = context;*cp != '\0';cp++)
		{
			gp = fontptr->getGlyph(*cp);
			if(gp)
			{
				if(*cp != ' ')
				{
					for(cc = 0;cc < gp->nContours();cc++)
					{
						sp = gp->getContourStartPoint(cc);
						ep = gp->getContourEndPoint(cc);
						//printf("(%i,%i)(%g,%g)\n",sp,ep,x,y);
						poc = 0;
						p[poc] = gp->getPoint(sp);
						poc++;
						for(pn = sp + 1;pn <= ep;pn++)
						{
							if(gp->isOnPoint(pn))
							{
								p[poc] = gp->getPoint(pn);
								poc++;
								drawSpline(v,m,p,poc,x,y);
								poc = 0;
								p[poc] = gp->getPoint(pn);
								poc++;
							}
							else
							{
								p[poc] = gp->getPoint(pn);
								poc++;
							}
						}
						p[poc] = gp->getPoint(sp);
						poc++;
						drawSpline(v,m,p,poc,x,y);
					}
				}
				x += gp->width();
			}
			x += offset(0);
			y += offset(1);
		}
	}

	if(this == v->getSelected()) v->setDrawSelected(0);

	return 0;
}

int	text::drawSpline(view *v,Matrix44 m,Vector2 *p,int pn,double x,double y)
{
	Vector3		pfa3,pfb3,pba3,pbb3;
	int		n;

	n = pn - 1;

/*
	int		ti,i,n;
	double		t,tc;
	Vector2		b;

	if(pn > 1)
	{
		pfb3 = Vector3(p[0](0) + x,p[0](1) + y,0);
		pbb3 = Vector3(p[0](0) + x,p[0](1) + y,thickness);
		for(ti = 1;ti <= pn * 3;ti++)
		{
			t = (double)ti / (double)(pn * 3);
			if(t <= 0.5)
			{
				tc = t / (1 - t);
				b = p[n];
				for(i = 1;i <= pn;i++)
				{
					b = b * ((double)i / (double)(pn - i + 1)) * tc + p[pn - i];
				}
				b = b * pow(1 - t,pn);
			}
			else
			{
				tc = (1 - t) / t;
				b = p[0];
				for(i = 1;i <= pn;i++)
				{
					b = b * ((double)i / (double)(pn - i + 1)) * tc + p[i - 1];
				}
				b = b * pow(t,pn);
			}
			pfa3 = pfb3;
			pba3 = pbb3;
			pfb3 = Vector3(b(0) + x,b(1) + y,0);
			pbb3 = Vector3(b(0) + x,b(1) + y,-thickness);
			v->drawLine(pfa3,pfb3,m);
			v->drawLine(pba3,pbb3,m);
			v->drawLine(pfa3,pba3,m);
		}
	}
*/

	pfa3 = Vector3(p[0](0) + x,p[0](1) + y,0);
	pfb3 = Vector3(p[n](0) + x,p[n](1) + y,0);

	pba3 = Vector3(p[0](0) + x,p[0](1) + y,thickness);
	pbb3 = Vector3(p[n](0) + x,p[n](1) + y,thickness);

	v->drawLine(pfa3,pfb3,m);
	v->drawLine(pba3,pbb3,m);
	v->drawLine(pfa3,pba3,m);


	return 0;
}

int	text::calculate()
{
	char		*cp;
	fontglyph	*gp = 0;
	double		x,y,h,m;

	x = 0;
	y = 0;
	h = 0;
	m = 0;

	for(cp = context;*cp != '\0';cp++)
	{
		gp = fontptr->getGlyph(*cp);
		if(gp)
		{
			x += gp->width();
			h = MAX(gp->ascend() + y,h);
			m = MIN(gp->descend() + y,m);
		}
		x += offset[0];
		y += offset[1];
	}

	vmin = Vector3(0,m,0);
	vmax = Vector3(x,h,thickness);

	return 0;
}
 
