#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "triangle.h"
#include "num.h" 
#include "view.h" 

triangle::triangle(base *p,char *n,
	double x1,double y1,double z1,
	double x2,double y2,double z2,
	double x3, double y3,double z3,
	int s,texture *t) :
		nonsolid(p,n,t)
{
	numtype = NUM_TRIANGLE;
	edge1 = Vector3(x1,y1,z1);
	edge2 = Vector3(x2,y2,z2);
	edge3 = Vector3(x3,y3,z3);
	n1 = Vector3(0,0,0);
	n2 = Vector3(0,0,0);
	n3 = Vector3(0,0,0);
	smooth=s;
	if(parent) addToParent(parent);
}

triangle::triangle(base *p,char *n,Vector3 &v1,Vector3 &v2,Vector3 &v3,texture *t) :
	nonsolid(p,n,t)
{
	numtype = NUM_TRIANGLE;
	edge1 = v1;
	edge2 = v2;
	edge3 = v3;
	n1 = Vector3(0,0,0);
	n2 = Vector3(0,0,0);
	n3 = Vector3(0,0,0);
	smooth = 0;

	if(parent) addToParent(parent); 
}

triangle::triangle(base *p,char *n,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	nonsolid(p,n,vs,vr,vt)
{
	numtype = NUM_TRIANGLE;
	edge1 = Vector3(1,0,0);
	edge2 = Vector3(0,1,0);
	edge3 = Vector3(0,0,1);
	n1 = Vector3(0,0,0);
	n2 = Vector3(0,0,0);
	n3 = Vector3(0,0,0);
	smooth = 0;

	if(parent) addToParent(parent); 
}

triangle::triangle(base *p,char *n,texture *t,Vector3 &vs,Vector3 &vr,Vector3 &vt) :
	nonsolid(p,n,t,vs,vr,vt)
{
	numtype = NUM_TRIANGLE;
	edge1 = Vector3(1,0,0);
	edge2 = Vector3(0,1,0);
	edge3 = Vector3(0,0,1);
	n1 = Vector3(0,0,0);
	n2 = Vector3(0,0,0);
	n3 = Vector3(0,0,0);
	smooth = 0;

	if(parent) addToParent(parent); 
}

triangle::triangle(base *p,triangle *tc) :
	nonsolid(p,(nonsolid*)tc)
{
	numtype = NUM_TRIANGLE;
	edge1 = tc->edge1;
	edge2 = tc->edge2;
	edge3 = tc->edge3;
	n1 = tc->n1;
	n2 = tc->n2;
	n3 = tc->n3;
	smooth = tc->smooth;

	if(parent) addToParent(parent); 
}

triangle::~triangle()
{
	removeFromParent();
}

base	*triangle::copy(base *p)
{
	return new triangle(p,this);
}

int	triangle::setNormal(Vector3 v1, Vector3 v2, Vector3 v3,int s)
{
	n1=v1;
	n2=v2;
	n3=v3;
	smooth=s;

	return 0;
}

int	triangle::setNormal(double x1, double y1, double z1, double x2, double y2, double z2, double x3, double y3, double z3, int s)
{
	n1 = Vector3(x1,y1,z1);
	n2 = Vector3(x2,y2,z2);
	n3 = Vector3(x3,y3,z3);
	smooth=s;

	return 0;
}

int	triangle::addToParent(base *p)
{
	if (!p) return -2;
	parent = p;
	return p->addChild(this);
}

int triangle::removeFromParent() 
{ 
	if (!parent) return -2;
	return parent->removeChild(this); 
}

void triangle::dumpNames(int tab, int)
{
	printTab(stderr,tab);
	printf("triangle: %s\n",name);
}

int	triangle::exportPOV(FILE *fp,int tab,int tabsize )
{
	printTab(fp,tab);
	fprintf(fp,"// Objectname = %s\n",name);
	printTab(fp,tab);
	fprintf(fp,"// Objecttype = triangle\n");

	printTab(fp,tab);
	if(!smooth)
	{
		fprintf(fp,"triangle {\n");
	}
	else
	{
		fprintf(fp,"smooth_triangle {\n");
	}

	printTab(fp,tab+tabsize);
	fprintf(fp,"<%g, %g, %g>, ",edge1[0],edge1[1],edge1[2]);
	if(smooth)
	{
		fprintf(fp,"<%f, %f, %f>, ",n1[0],n1[1],n1[2]);
	}
	fprintf(fp,"\n");

	printTab(fp,tab+tabsize);
	fprintf(fp,"<%g, %g, %g>, ",edge2[0],edge2[1],edge2[2]);
	if(smooth)
	{
		fprintf(fp,"<%f, %f, %f>, ",n1[0],n1[1],n1[2]);
	}
	fprintf(fp,"\n");

	printTab(fp,tab+tabsize);
	fprintf(fp,"<%g, %g, %g> ",edge3[0],edge3[1],edge3[2]);
	if(smooth)
	{
		fprintf(fp,", <%g, %g, %g> ",n1[0],n1[1],n1[2]);
	}
	fprintf(fp,"\n");

	if(texptr) texptr->exportPOV(fp,tab + tabsize,tabsize);

	dim::exportPOV(fp,tab,tabsize);

	printTab(fp,tab);
	fprintf(fp,"}\n\n");	

	return 0;
}

int triangle::save(FILE *fp)
{
	if (!fp) return -1;
	setFile(fp);
	
	if(smooth)
	{
		writeChunk("TRIA");
		SN(name);
		SV(edge1);
		SV(edge2);
		SV(edge3);
		SI(1);
		SV(n1);
		SV(n2);
		SV(n3);
	}
	else
	{
		writeChunk("TRIA");
		SN(name);
		SV(edge1);
		SV(edge2);
		SV(edge3);
		SI(0);
	}
	dim::save(fp);
	saveTexture(fp);
	writeChunkLen();

	return 0;
}

int triangle::load(FILE *fp,int l)
{
	int		m = 1;

	edge1 = Vector3(readDouble(fp),readDouble(fp),readDouble(fp));
	edge2 = Vector3(readDouble(fp),readDouble(fp),readDouble(fp));
	edge3 = Vector3(readDouble(fp),readDouble(fp),readDouble(fp));
	smooth=readInt(fp);
	if(smooth)
	{
		n1 = Vector3(readDouble(fp),readDouble(fp),readDouble(fp));
		n2 = Vector3(readDouble(fp),readDouble(fp),readDouble(fp));
		n3 = Vector3(readDouble(fp),readDouble(fp),readDouble(fp));
		m = 2;
	}

	dim::load(fp,l - 9 * sizeof(double) * m);
 	if(l > (int)(sizeof(double) * (m * 9 + 9)))
	{
		loadTexture(fp,l - sizeof(double) * (m * 9 + 9));
	}

	return 0;
}

int	triangle::draw(view *v,Matrix44 m)
{
	Vector3	p1,p2,p3;

	//printf("Triangle\n");

	if(isFlag(HIDE)) return 0;

	p1 = edge1;
	p2 = edge2;
	p3 = edge3;

	dimMatrix(m);

	if(this == v->getSelected()) v->setDrawSelected(1);

	if(isFlag(DRAW_BOUNDINGBOX))
		drawBB(v,m);
	else
	{
		v->drawLine(p1,p2,m);
		v->drawLine(p2,p3,m);
		v->drawLine(p3,p1,m);
	}

	if(this == v->getSelected()) v->setDrawSelected(0);

	return 0;
}

int	triangle::calculate()
{
	vmin[0] = MIN3(edge1[0],edge2[0],edge3[0]);
	vmin[1] = MIN3(edge1[1],edge2[1],edge3[1]);
	vmin[2] = MIN3(edge1[2],edge2[2],edge3[2]);
	vmax[0] = MAX3(edge1[0],edge2[0],edge3[0]);
	vmax[1] = MAX3(edge1[1],edge2[1],edge3[1]);
	vmax[2] = MAX3(edge1[2],edge2[2],edge3[2]);

	return 0;
}


