#include "color.h"
#include <chunk.h>

color::color(double rr,double gg,double bb,double tt,double vv)
{
	r = rr;
	g = gg;
	b = bb;
	t = tt;
	v = vv;
}

color::~color()
{
}

double	color::red()
{
	return r;
}

double	color::green()
{
	return g;
}

double	color::blue()
{
	return b;
}

double	color::transparency()
{
	return t;
}

double	color::value()
{
	return v;
}

void	color::setRed(double rr)
{
	r = MIN(1,MAX(0,rr));
}

void	color::setGreen(double gg)
{
	g = MIN(1,MAX(0,gg));
}

void	color::setBlue(double bb)
{
	b = MIN(1,MAX(0,bb));
}

void	color::setTranparency(double tt)
{
	t = MIN(1,MAX(0,tt));
}

void	color::setValue(double vv)
{
	v = MIN(1,MAX(0,vv));
}

int		color::save(FILE *fp)
{
	if(!fp)	return -1;

	SD(r);
	SD(g);
	SD(b);
	SD(t);
	SD(v);

	return 0;
}

int		color::load(FILE *fp,int l)
{
	if(!fp) return -1;

	if(l < (int)(5 * sizeof(double))) return -2;

	fread(&r,1,sizeof(double),fp);
	fread(&g,1,sizeof(double),fp);
	fread(&b,1,sizeof(double),fp);
	fread(&t,1,sizeof(double),fp);
	fread(&v,1,sizeof(double),fp);

	return 0;
}

int		color::exportPOV(FILE *fp,int tab,int,int with_tr,int as_map)
{
	if(!fp) return -1;

	printTab(fp,tab);

	if(as_map)
		fprintf(fp,"[%g ",v);

	if(with_tr)
		fprintf(fp,"color rgbt<%g,%g,%g,%g>",r,g,b,t);
	else
		fprintf(fp,"color rgb<%g,%g,%g>",r,g,b);

	if(as_map)
		fprintf(fp,"]");

	fprintf(fp,"\n");

	return 0;
}


colormap::colormap() :
	list<color>()
{
}

void	colormap::addColor(color c)
{
	int		t;

	for(t = 0;t < length();t++)
	{
		if(c.value() > at(t).value())
		{
			insertAt(c,t);
			return;
		}
	}
	append(c);
}

void	colormap::removeColor(int n)
{
	if(n <0 || n >= length()) return;

	deleteAt(n);
}

color	colormap::getColor(double v)
{
	color		cmv,l,r;
	int		t;
	double		dl,dr;

	if(v < 0) v = 0;
	if(v > 1) v = 1;

	switch(length())
	{
		case 0:
			r.setValue(v);
			return r;
		case 1:
			r = at(0);
			r.setValue(v);
			return r;
	}
	l = at(0);
	r = at(length() - 1);

	if(v <= l.value())
	{
		l.setValue(v);
		return l;
	}
	if(v >= r.value())
	{
		r.setValue(v);
		return r;
	}

	for(t = 0;t < length() && v > at(t).value();t++);

	l = at(t - 1);
	r = at(t);

	dl = 1 - (v - l.value()) / (r.value() - l.value());
	dr = 1 - (r.value() - v) / (r.value() - l.value());

	cmv.setRed(r.red() * dr + l.red() * dl);
	cmv.setGreen(r.green() * dr + l.green() * dl);
	cmv.setBlue(r.blue() * dr + l.blue() * dl);
	cmv.setValue(v);

	return cmv;
}

int	colormap::getColorIndex(double v)
{
	color		l,r;
	int		t;

	if(v < 0) v = 0;
	if(v > 1) v = 1;

	switch(length())
	{
		case 0:
			return -1;
		case 1:
			return 0;
	}

	l = at(0);
	r = at(length() - 1);

	if(v <= l.value())
	{
		return 0;
	}
	if(v >= r.value())
	{
		return length() - 1;
	}

	for(t = 0;t < length() && v > at(t).value();t++);

	return t - 1;
}

int	colormap::entries()
{
	return length();
}

color	&colormap::getColor(int n)
{
	return at(n);
}

int		colormap::save(FILE *fp)
{
	int	i;

	if(!fp) return -1;

	for(i = 0;i < length();i++)
	{
		at(i).save(fp);
	}

	return 0;
}

int		colormap::load(FILE *fp,int l)
{
	color	c;
	int	i;

	if(!fp) return -1;

	for(i = 0;l > i * (int)(sizeof(double) * 5 + 8);i++)
	{
		c.load(fp,l);
		append(c); 
	}

	return 0;
}

int		colormap::exportPOV(FILE *fp,int tab,int tabsize)
{
	int		i,t;

	if(fp == 0) return -1;

	if(length() == 0) return 0;
	if(length() == 1)
		return getFirst().exportPOV(fp,tab,tabsize);

	printTab(fp,tab);
	fprintf(fp,"color_map\n");
	printTab(fp,tab);
	fprintf(fp,"{\n");

	t = 0;
	for(i = 0;i < length();i++)
	{
		if(at(i).transparency())
		{
			t = 1;
			break;
		}
	}

	for(i = 0;i < length();i++)
	{
		at(i).exportPOV(fp,tab + tabsize,tabsize,t,1);
	}

	printTab(fp,tab);
	fprintf(fp,"}\n");

	return 0;
}

void	colormap::sort()
{
	int		t,i;
	color		c;

	for(t = 0;t < length() - 1;t++)
	{
		for(i = t + 1;i < length();i++)
		{
			if(at(t).value() > at(i).value())
			{
				c = at(t);
				at(t) = at(i);
				at(i) = c;
			}
		}
	}
}





