#include "pigment.h"

static char	fnname[pigment::NFunctions][20] =
		{
			"checker",
			"marble",
			"wood",
			"granite",
			"leopard",
			"onion",
			"brick",
			"hexagon",
			"agate",
			"average",
			"crackle",
			"dents",
			"mandel",
			"waves",
			"spotted",
			"ripples",
			"gradient",
			"radial",
			"spiral1",
			"spiral2",
			"bozo",
			"bumps",
			"wrinkles",
		};

pigment::pigment() :
	chunk()
{
	fn = Marble;
	turb = 0;
}

void	pigment::setFunction(FUNCTION f)
{
	fn = f;
}

void	pigment::setTurbulence(double t)
{
	turb = MAX(0,t);
}

int	pigment::function()
{
	return fn;
}

double	pigment::turbulence()
{
	return turb;
}

char	*pigment::getFunctionName(FUNCTION n)
{
	return fnname[n];
}

int	pigment::load(FILE *fp,int l)
{
	if(!fp) return -1;

	fread(&fn,1,sizeof(FUNCTION),fp);
	fread(&turb,1,sizeof(double),fp);

	colormap::load(fp,l - sizeof(FUNCTION) - sizeof(double));

	return 0;
}

int	pigment::save(FILE *fp)
{
	if(!fp) return -1;

	setFile(fp);

	writeChunk("PGMT");

	fwrite(&fn,1,sizeof(FUNCTION),fp);
	SD(turb);

	colormap::save(fp);

	writeChunkLen();

	return 0;
}

int	pigment::exportPOV(FILE *fp,int tab,int tabsize)
{
	if(fp == 0) return -1;

	if(length() == 0) return -2;

	printTab(fp,tab);
	fprintf(fp,"pigment\n");

	printTab(fp,tab);
	fprintf(fp,"{\n");

	if(length() > 1)
	{
		printTab(fp,tab + tabsize);
		fprintf(fp,"%s\n",fnname[fn]);
	}

	colormap::exportPOV(fp,tab + tabsize,tabsize);

	if(length() > 1)
	{
		printTab(fp,tab + tabsize);
		fprintf(fp,"turbulence %g\n",turb);
	}

	printTab(fp,tab);
	fprintf(fp,"}\n");

	return 0;
}



