#include "texture.h"
#include <misc.h>

texture::texture(char *n,double rr,double gg,double bb) :
	pigment()
{
	color		cl(rr,gg,bb,0,0);

	name = strdup(n);

	insert(cl);

	colorMapMapping = MAP_PLANAR;
	colorMapName = strdup("");
}

texture::texture(char *n) :
	pigment()
{
	name = strdup(n);

	colorMapMapping = MAP_PLANAR;
	colorMapName = strdup("");
}

texture::~texture()
{
	if(name) free(name);
	if(colorMapName) free(colorMapName);
}

int	texture::setName(char *n)
{
	if(name) free(name);
	name = strdup(n);
	return 0;
}

char	*texture::getName()
{
	return name;
}

void texture::setColorMapMapping(int x)
{
	colorMapMapping=x;
}

int texture::setColorMapName(char *n)
{
	colorMapName = strdup(n);

	return 0;
}

int	texture::save(FILE *fp)
{
	pigment		tmp = *this;

	if(!fp) return -1;

	setFile(fp);

	writeChunk("TXTR");

	SN(name);

	tmp.save(fp);

	writeChunkLen();

	return 0;
}

int	texture::load(FILE *fp,int l)
{
	char	chunk[4];
	int	len;

	if(!fp) return -1;

	setFile(fp);

	name = readName(fp);

	fread(chunk,1,4,fp);
	fread(&len,1,4,fp);

	if(strncmp(chunk,"PGMT",4) == 0)
		pigment::load(fp,len);

	return 0;
}

int	texture::savePtr(FILE *fp)
{
	if(!fp) return -1;

	setFile(fp);
	writeChunk("TXTP");

	SN(name);

	writeChunkLen();

	return 0;
}

int	texture::loadPtr(FILE *fp,int)
{
	if(!fp) return -1;

	setFile(fp);

	name = readName(fp);

	return 0;
}

int	texture::exportPOV(FILE *fp,int tab,int tabsize)
{
	printf("texture: %i colors\n",length());

	printTab(fp,tab);
	fprintf(fp,"// Objectname = %s\n",name);
	printTab(fp,tab);
	fprintf(fp,"// Objecttype = texture\n");
	printTab(fp,tab);

	pigment::exportPOV(fp,tab,tabsize); 
	// dim::exportPOV(fp,tab + tabsize,tabsize);

	return 0;
}


