// texturemap.cpp
//
// Diese Klasse ist nur eine virtuelle Klasse (z.B. fr KDE)
// Sie ermglicht eine Hierachische Katalog mit mehreren Seiten
// So knnte so ein Katalog so aussehen:
//
// Basiskatalog
//  - POV-katalog
//   - Stein
//   - Holz
//  - k3de-katalog
//   - Stein
//   - Holz
//   - Haut
//   - Transpatenz
//  - eigener Katalog
//   - Stein
//   - Holz
//   - Haut

#include "texturemap.h"

texturemap::texturemap(char *n)
{
	_name = strdup(n);
}

texturemap::~texturemap()
{
	if(_name) free(_name);
}

char	*texturemap::name()
{
	return _name;
}

texture	*texturemap::findTexture(char *n)
{
	int	i;
	texture	*t;

	for(i = 0;i < texturelist.length();i++)
	{
		if(strcmp(texturelist[i]->getName(),n) == 0)
			return texturelist[i];
	}
	for(i = 0;i < texturemaplist.length();i++)
	{
		t = texturemaplist[i]->findTexture(n);
		if(t != 0)
			return t;
	}

	return 0;
}

char	*texturemap::texturePath(char *n)
{
	int	i;
	char	*cp,*str;

	for(i = 0;i < texturelist.length();i++)
	{
		if(strcmp(texturelist[i]->getName(),n) == 0)
			return strdup(texturelist[i]->getName());
	}
	for(i = 0;i < texturemaplist.length();i++)
	{
		cp = texturemaplist[i]->texturePath(n);
		if(cp != 0)
		{
			str = (char*)malloc(strlen(cp) + strlen(_name) + 2);
			strcpy(str,_name);
			strcat(str,"/");
			strcat(str,cp);

			free(cp);

			return str;
		}
	}

	return 0;
}


void	texturemap::addTexture(texture *t)
{
	texturelist.append(t);
}

void	texturemap::addTexturemap(texturemap *tm)
{
	texturemaplist.append(tm);
}

void	texturemap::removeTexture(texture *t)
{
	if(texturelist.find(t))
		texturelist.deleteCurrent();
}

void	texturemap::removeTexturemap(texturemap *tm)
{
	if(texturemaplist.find(tm))
		texturemaplist.deleteCurrent();
}

int	texturemap::getNumTextures()
{
	return texturelist.length();
}

texture	*texturemap::getTexture(int n)
{
	return texturelist[n];
}

texture	*texturemap::getFirstTexture()
{
	return texturelist.getFirst();
}

texture	*texturemap::getNextTexture()
{
	return ++texturelist;
}

int	texturemap::getNumTextureMaps()
{
	return texturemaplist.length();
}

texturemap	*texturemap::getTextureMap(int n)
{
	return texturemaplist[n];
}

texturemap	*texturemap::getFirstTexturemap()
{
	return texturemaplist.getFirst();
}

texturemap	*texturemap::getNextTexturemap()
{
	return ++texturemaplist;
}

int		texturemap::save(FILE *fp)
{
	int	n;

	if(!fp) return -1;

	setFile(fp);

	writeChunk("TXTM");

	SN(_name);

	for(n = 0;n < texturelist.length();n++)
	{
		texturelist[n]->save(fp);
	}
	for(n = 0;n < texturemaplist.length();n++)
	{
		texturemaplist[n]->save(fp);
	}

	writeChunkLen();

	return 0;
}

int	texturemap::load(FILE *fp,int l)
{
	int	pos,len;
	char	chunk[4];

	if(!fp) return -1;

	setFile(fp);

	pos = ftell(fp);	

	_name = readName();

	while(l > ftell(fp) - pos)
	{
		fread(chunk,1,4,fp);
		fread(&len,1,sizeof(int),fp);

		if(strncmp(chunk,"TXTM",4) == 0)
		{
			texturemap	*tm;

			tm = new texturemap("");
			tm->load(fp,len);
			texturemaplist += tm;
		}
		else if(strncmp(chunk,"TXTR",4) == 0)
		{
			texture		*t;

			t = new texture("");
			t->load(fp,len);
			texturelist += t;
		}
		else
			fseek(fp,len,SEEK_CUR);
	}

	return 0;
}




