#include "dialogs.h"
#include "dialogs.moc"

#include <klocale.h>

#define stloc(a)	((char*)locale->translate((a)))


TextureDialog::TextureDialog(QWidget *parent,const char *name,texture *t) :
	QDialog(parent,name,TRUE)
{
	txtptr=t;


	namelabel = new QLabel(this);
	namelabel->setText("Name:");

	nameline = new QLineEdit(this);

	namelabel->setGeometry(5,5,100,25);
	nameline->setGeometry(110,5,200,25);

	tabdialog = new QTabDialog(this);

	color = new TextureColor(tabdialog,name,t);
		
	tabdialog->addTab(color,"Color");
	
	tabdialog->setGeometry(10,40,420,240);
	tabdialog->setCancelButton();

	connect( tabdialog, SIGNAL(cancelButtonPressed()), SLOT(reject()));
	connect( tabdialog, SIGNAL(applyButtonPressed()), SLOT(accept()));

	color->setGeometry(5,35,tabdialog->width() - 10,tabdialog->height() - 40);


	resize(440,300);
	setMinimumSize(440,300);
}

TextureDialog::~TextureDialog()
{
	delete	color;
	delete  tabdialog;
}

texture	*TextureDialog::getTexture()
{
	txtptr->setName( (char*)nameline->text());

	color->getTexture();

	//txtptr->setColorMapMapping(color->getMapping() );
	//txtptr->setColorMapName(color->getMapName() );

	return txtptr;
}

void	 TextureDialog::paintEvent(QPaintEvent*)
{
	int		w = width();
	int		h = height();
	QSize		ls = namelabel->sizeHint();

	namelabel->setGeometry(5,5,ls.width(),25);
	nameline->setGeometry(10 + ls.width(),5,(w - 15) - ls.width(),25);
	tabdialog->setGeometry(5,40,w - 10,h - 40);
}

// ******************************************************************************************

NameDialog::NameDialog(QWidget *parent=0,const char *name=0) :
	QDialog(parent,name,TRUE)
{
	KLocale	*locale;

	setCaption(name);

	locale = new KLocale();

	line = new QLineEdit(this);

	ok = new QPushButton( this,stloc("Ok"));
	ok->setText(stloc("OK"));
	connect( ok, SIGNAL(clicked()), SLOT(accept()) );

	cancel = new QPushButton( this,stloc("Cancel") );
	cancel->setText(stloc("Cancel"));
	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

	setMaximumSize(150,65);
	QDialog::resize(150,65);
	setMinimumSize(150,65);

	line->setGeometry(5,5,140,25);
	cancel->setGeometry(5,35,67,25);
	ok->setGeometry(77,35,67,25);	

	delete locale;
}

NameDialog::~NameDialog()
{
	delete	ok;
	delete	cancel;
	delete	line;
}

QString	NameDialog::getName()
{
	return line->text();
}

void	NameDialog::setName(QString s)
{
	line->setText(s);
}

// ******************************************************************************************

BlobDialog::BlobDialog(QWidget *parent=0,const char *name=0) :
	QDialog(parent,name,TRUE)
{
	KLocale	*locale;
	int	w,w2;

	setCaption(name);

	locale = new KLocale();

	label = new QLabel(this);
	label->setText("Thresholdvalue:");

	line = new QLineEdit(this);

	ok = new QPushButton( this,stloc("Ok"));
	ok->setText(stloc("OK"));
	connect( ok, SIGNAL(clicked()), SLOT(accept()) );

	cancel = new QPushButton( this,stloc("Cancel") );
	cancel->setText(stloc("Cancel"));
	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

	w2 = label->sizeHint().width();
	w = w2 * 3 / 2 + 15;

	setMaximumSize(w,65);
	QDialog::resize(w,65);
	setMinimumSize(w,65);

	label->setGeometry(5,5,w2,25);
	line->setGeometry(10 + w2,5,w2 / 2,25);

	w2 = w2 * 2 / 3;
	cancel->setGeometry((w - 5) / 2 - w2,35,w2,25);
	ok->setGeometry((w - 5) / 2 + 5,35,w2,25);	

	delete locale;
}

BlobDialog::~BlobDialog()
{
}

double	BlobDialog::thresholdValue()
{
	QString	str;

	str = line->text();

	return str.toDouble();
}

void	BlobDialog::setThresholdValue(double t)
{
	QString	str;

	str.setNum(t);
	line->setText(str);
}

// ******************************************************************************************


DimDialog::DimDialog(QWidget *parent,const char *name) :
	QDialog(parent,name,TRUE)
{
	KLocale	*locale;

	setCaption(name);

	dimw = new DimWidget(this,"");

	locale = new KLocale();

	ok = new QPushButton( this,stloc("Ok"));
	ok->setText(stloc("OK"));
	connect( ok, SIGNAL(clicked()), SLOT(accept()) );

	cancel = new QPushButton( this,stloc("Cancel") );
	cancel->setText(stloc("Cancel"));
	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

	setMaximumSize(300,140);
	QDialog::resize(300,140);
	setMinimumSize(300,140);

	dimw->setGeometry(0,0,300,125);
	cancel->setGeometry(67,110,80,25);
	ok->setGeometry(153,110,80,25);	

	delete locale;

	dimw->layout();
}

DimDialog::~DimDialog()
{
	delete	ok;
	delete	cancel;
	delete	dimw;
}

void	DimDialog::save(Vector3 &s,Vector3 &r,Vector3 &t)
{
	dimw->save(s,r,t);
}

void	DimDialog::load(Vector3 s,Vector3 r,Vector3 t)
{
	dimw->load(s,r,t);
}

// ******************************************************************************************

TextDialog::TextDialog(QWidget *parent,const char *name) :
	QDialog(parent,name,TRUE)
{
	KLocale	*locale;

	locale = new KLocale();

	setCaption(name);

	lcontext = new QLabel(this);
	lcontext->setText(stloc("Context:"));

	wcontext = new QLineEdit(this);

	lfontname = new QLabel(this);
	lfontname->setText(stloc("Fontname:"));

	wfontname = new QLineEdit(this);

	lthickness = new QLabel(this);
	lthickness->setText(stloc("Thickness:"));

	wthickness = new QLineEdit(this);

	loffset = new QLabel(this);
	loffset->setText(stloc("Offset:"));

	wxoffset = new QLineEdit(this);
	wyoffset = new QLineEdit(this);

	
	ok = new QPushButton( this,stloc("Ok"));
	ok->setText(stloc("OK"));
	connect( ok, SIGNAL(clicked()), SLOT(accept()) );

	cancel = new QPushButton( this,stloc("Cancel") );
	cancel->setText(stloc("Cancel"));
	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );

	setMaximumSize(300,170);
	QDialog::resize(300,170);
	setMinimumSize(300,170);

	lcontext->setGeometry(5,5,140,25);
	wcontext->setGeometry(145,5,150,25);

	lfontname->setGeometry(5,30,140,25);
	wfontname->setGeometry(145,30,150,25);

	lthickness->setGeometry(5,60,140,25);
	wthickness->setGeometry(145,60,150,25);

	loffset->setGeometry(5,90,140,25);
	wxoffset->setGeometry(145,90,74,25);
	wyoffset->setGeometry(221,90,74,25);

	cancel->setGeometry(67,130,80,25);
	ok->setGeometry(153,130,80,25);	

	delete locale;
}

TextDialog::~TextDialog()
{
	delete	ok;
	delete	cancel;
	delete	wcontext;
	delete	wfontname;
	delete	wthickness;
	delete	wxoffset;
	delete	wyoffset;
	delete	lcontext;
	delete	lfontname;
	delete	lthickness;
	delete	loffset;
}

void	TextDialog::save(QString &context,QString &fontname,double &thickness,Vector2 &offset)
{
	float		d;
	const char	*str;

	context = wcontext->text();

	fontname = wfontname->text();

	str = wthickness->text();
	sscanf(str,"%g",&d);
	thickness = d;

	str = wxoffset->text();
	sscanf(str,"%g",&d);
	offset[0] = d;
	str = wyoffset->text();
	sscanf(str,"%g",&d);
	offset[1] = d;
}

void	TextDialog::load(QString context,QString fontname,double thickness,Vector2 offset)
{
	char		str[128];

	wcontext->setText(context);

	wfontname->setText(fontname);

	sprintf(str,"%g",thickness);
	wthickness->setText(str);

	sprintf(str,"%g",offset[0]);
	wxoffset->setText(str);
	sprintf(str,"%g",offset[1]);
	wyoffset->setText(str);
}






