#include "texturetable.h"
#include "texturetable.moc"

#include <kprocess.h>
#include <qpixmap.h>

TextureTableItem::TextureTableItem(TextureTable *parent,int id,const char *title,QPixmap &p,texture *t,int add) :
	SelectTableItem((SelectTable*)parent,id,title,p)
{
	texptr = t;
	if(parent && t && add)
	{
		parent->getTextureMap()->addTexture(t);
	}
}

TextureTableItem::~TextureTableItem()
{
}

void		TextureTableItem::setTexture(const texture *t)
{
	texptr = t;
}

const texture	*TextureTableItem::getTexture() const
{
	return texptr;
}

//********************************************************************

TextureTable::TextureTable(QWidget *parent,const char *name,WFlags f,texturemap *t) :
	SelectTable(parent,name,f)
{
	tmap = t;

	connect(this,SIGNAL(selected(int)),this,SLOT(selectedTexture(int)));

	// scandir("data/texture");

	//new TextureTableItem(this,-1,"Crackle",* new QPixmap("data/texture/crackle.ppm"),new texture("red",1,0,0));
	//new TextureTableItem(this,-1,"Marble",* new QPixmap("data/texture/marble.ppm"),new texture("blue",0,0,1));
	new TextureTableItem(this,-1,"None",* new QPixmap("data/texture/none.ppm"),0);
}

TextureTable::~TextureTable()
{
}

bool		TextureTable::insertItem(TextureTableItem *i)
{
	bool	ok;

	ok = itemlist.insert(i->id(),i);

	if(ok)
	{
		if(tmap) tmap->addTexture((texture*)i->getTexture());
		else
			printf("TextureTable: error no texturemap\n");

		if((unsigned)(numRows() * numCols()) < itemlist.count())
		{
			if(numCols() * cellWidth() < width())
			{
				setNumCols(numCols() + 1);
				return ok;
			}
			if(numRows() * cellHeight() < height())
			{
				setNumRows(numRows() + 1);
				return ok;
			}
			if(numRows() < numCols())
			{
				setNumRows(numRows() + 1);
			}
			else
			{
				setNumCols(numCols() + 1);
			}
		}
	}

	return ok;
}

TextureTableItem	*TextureTable::item(int i)
{
	TextureTableItem	*tti;

	printf("itemlist: %i\n",itemlist.count());

	tti = (TextureTableItem*)itemlist.first();
	while(tti)
	{
		printf("item: (%i) = (%i)\n",i,tti->id());

		if(tti->id() == i) return tti;
		tti = (TextureTableItem*)itemlist.next();
	}

	return NULL;
}

void	TextureTable::selectedTexture(int i)
{
	TextureTableItem	*tti;
	texture			*t;

	printf("%i\n",i);

	tti = item(i);

	if(tti)
	{
		t = (texture*)tti->getTexture();
		emit setTexture(t);
	}
}

void	TextureTable::setTextureMap(texturemap *tm)
{
	tmap = tm;
	if(tmap)
	{
		int		n;
		texture		*t;
		texturemap	*tm;
		KProcess	p;
		char		tname[1024];
		char		fnppm[1024];
		char		com[2048];
		int		i;
		FILE		*fp;
		QPixmap		*pm;

		itemlist.clear();
		new TextureTableItem(this,-1,"None",* new QPixmap("data/texture/none.ppm"),0);

		for(n = 0;n < tmap->getNumTextures();n++)
		{
			t = tmap->getTexture(n);
			sprintf(tname,"%s.tex",t->getName());
			sprintf(fnppm,"data/texture/%s.ppm",t->getName());

			for(i = 0;fnppm[i] != '\0';i++)
			{
				if(fnppm[i] == ' ') fnppm[i] = '_';
			}

			fp = fopen("data/texture/texture.pov","w");
			t->exportPOV(fp,0,4);
			fclose(fp);

			//sprintf(com,"povray +Ldata/texture +FP +W100 +H100 +Itexturebody.pov +O%s",fnppm);
			//p.setExecutable(com);

			sprintf(com,"+O%s",fnppm);
			p << "povray" << "+Ldata/texture" << "+FP" << "+W100" << "+H100" << "+Itexturebody.pov" << com;
			p.start(KProcess::Block,KProcess::NoCommunication);
	
			pm = new QPixmap(fnppm);

			new TextureTableItem(this,-1,t->getName(),*pm,t,0);
		}
		for(n = 0;n < tmap->getNumTextureMaps();n++)
		{
			tm = tmap->getTextureMap(n);
			// Please implement this
		}
		repaint();
	}
}

texturemap	*TextureTable::getTextureMap()
{
	return tmap;
}


