#include "colormapwidget.h"
#include "colormapwidget.moc"

#include <qslider.h>


ColorMapWidget::ColorMapWidget(QWidget *parent) :
	QWidget(parent)
{
	QColor		qc;

	qc.setRgb(255,0,0);

	value = new QSpinBox(0,0,1,this);

	add = new QPushButton(this);
	add->setText("Add");

	del = new QPushButton(this);
	del->setText("Del");

	slider = new QSlider(QSlider::Horizontal,this);

	colorbt = new KColorButton(qc,this);

	connect(add,SIGNAL(clicked()),this,SLOT(addColorValue()));
	connect(del,SIGNAL(clicked()),this,SLOT(delColorValue()));
	connect(slider,SIGNAL(valueChanged(int)),this,SLOT(changeValue(int)));
	connect(value,SIGNAL(valueChanged(int)),this,SLOT(changeEntry(int)));
	connect(colorbt,SIGNAL(changed(const QColor&)),this,SLOT(changeColor(const QColor&)));

	colorflow = new ColorFlowWidget(this);
}

ColorMapWidget::~ColorMapWidget()
{
}

void ColorMapWidget::paintEvent(QPaintEvent*)
{
	int w,h,x,y;

	x = 0;
	y = 0;
	h = 25;
	w = 50;
	value->setGeometry(x,y,w,h);

	x += w + 5;
	colorbt->setGeometry(x,y,w,h);

	x += w + 5;
	w = width() - x - 5;
	colorflow->setGeometry(x,y,w,h);

	x = 0;
	y += h + 5;
	w = 50;
	add->setGeometry(x,y,w,h);
	x += w + 5;
	del->setGeometry(x,y,w,h);

	x += w + 5;
	w = colorflow->width();
	slider->setGeometry(x,y,w,h);
	slider->setRange(0,w - 4);

	changeEntry(value->value());
}

void	ColorMapWidget::addColorValue()
{
	color		c(0,0,0,0,slidervalue);

	colorflow->addColor(c);
	value->setRange(0,value->maxValue() + 1);

	colorflow->paint();
}

void	ColorMapWidget::delColorValue()
{
	if(value->maxValue() == 0) return;

	colorflow->removeColor(value->value());
	value->setRange(0,value->value() - 1);

	colorflow->paint();
}

void	ColorMapWidget::changeValue(int v)
{
	int	n;
	double	d;

	if(slidervalue == v) return;
	slidervalue = v;

	n = value->value();
	d = (double)slidervalue / (double)slider->width();

	if(d > colorflow->getColor(n).value() && d >= colorflow->getColor(n + 1).value())
	{
		colorflow->getColor(n).setValue(colorflow->getColor(n + 1).value());
		colorflow->getColor(n + 1).setValue(d);
		value->setValue(n + 1);
	}
	else if(d < colorflow->getColor(n).value() && d <= colorflow->getColor(n - 1).value())
	{
		colorflow->getColor(n).setValue(colorflow->getColor(n - 1).value());
		colorflow->getColor(n - 1).setValue(d);
		value->setValue(n - 1);
	}
	else
		colorflow->getColor(n).setValue(d);

//	colorflow->sort();
	colorflow->paint();
}

void	ColorMapWidget::changeEntry(int v)
{
	color		c;
	int		r,g,b;

	c = colorflow->getColor(v);

	r = (int)(c.red() * 255);
	g = (int)(c.green() * 255);
	b = (int)(c.blue() * 255);

	colorbt->setColor(QColor(r,g,b));
	slider->setValue((int)(c.value() * slider->width()));

	colorflow->paint();
}

void	ColorMapWidget::changeColor(const QColor &nc)
{
	colorflow->getColor(value->value()).setRed((double)nc.red() / 255.0);
	colorflow->getColor(value->value()).setGreen((double)nc.green() / 255.0);
	colorflow->getColor(value->value()).setBlue((double)nc.blue() / 255.0);

	colorflow->paint();
}

void		ColorMapWidget::addColor(color c)
{
	colorflow->addColor(c);
}

color		ColorMapWidget::getColor(int n)
{
	color	c;

	c = colorflow->getColor(n);
	return c;
}

int		ColorMapWidget::nColors()
{
	return colorflow->length();
}


