#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "font.h"


font::font(char *n,char *fn,char *fp)
{
	name = strdup(n);
	if(fn) filename = strdup(fn);
	else filename = 0;
	if(fp) pathname = strdup(fp);
	else pathname = 0;

	glyphcache = 0;

	fontptr = 0;

	initialize();
}

font::~font()
{
	if(name) free(name);
	if(filename) free(filename);
	if(pathname) free(pathname);

	destroy();
}

fontglyph	*font::getGlyph(char *str)
{
	GLYFPtr		glyfptr;
	fontglyph	*g;
	int		n;
	Vector2		v2;
	double		x,y,xo,yo;

	if((g = getCachedGlyph(str)) != 0)
	{
//		printf("%s cached\n",str);
		return g;
	}
	printf("%s not cached\n",str);

	if(fontptr == 0)
	{
		if(initialize() != 0)
		{
			printf("Cannot initialize font\n");

			return 0;
		}
	}

	g = new fontglyph(str);

	if(strlen(str) == 1)
		glyfptr = ttfLoadGlyphIndex(fontptr,(int)str[0] - 65 + 36);
	else if(strcmp(str,"Space") == 0)
		glyfptr = ttfLoadGlyphIndex(fontptr,3);

	xo = 0;
	yo = 0;

	for(n = 0;n < glyfptr->instructionLength;n++)
	{
		x = (double)glyfptr->xCoordinates[n] / 2048.0;
		y = (double)glyfptr->yCoordinates[n] / 2048.0;
		xo += x;
		yo += y;
		v2 = Vector2(xo,yo);
		g->addPoint(v2,glyfptr->flags[n]);
	}
	for(n = 0;n < glyfptr->numberOfContours;n++)
	{
		g->addContour(glyfptr->endPtsOfContours[n]);
	}

	cacheGlyph(g);

	return g;
}

fontglyph	*font::getGlyph(char c)
{
	char	str[24];

	switch(c)
	{
		case ' ':
			strcpy(str,"Space");
		break;
		case '\t':
			strcpy(str,"Tab");
		break;
		case '\n':
			strcpy(str,"Carrige Return");
		break;
		default:
			str[0] = c;
			str[1] = '\0';
	}

	// cmap-lookup follows later
	return getGlyph(str);
}

int		font::initialize()
{
	char		fn[2048];

	destroy();

	fn[0] = '\0';
	if(pathname)
	{
		strcpy(fn,pathname);
		strcat(fn,"/");
	}
	if(filename) strcat(fn,filename);
	else
	{
		strcat(fn,name);
		strcat(fn,".ttf");
	}
	printf("Open font %s\n",fn);
	fontptr = ttfInitFont(fn);

	return !fontptr;
}

void		font::destroy()
{
	if(fontptr) ttfFreeFont(fontptr);
	fontptr = 0;

	if(glyphcache)
	{
		int	n;

		for(n = 0;glyphcache[n] != 0;n++)
			delete glyphcache[n];
		free(glyphcache);
		glyphcache = 0;
	}
}

fontglyph	*font::getCachedGlyph(char *str)
{
	fontglyph	*g;
	int		n;

	if(glyphcache == 0) return 0;

	for(n = 0;glyphcache[n] != 0;n++)
	{
		g = glyphcache[n];
		if(g->getName() != 0)
			if(strcmp(str,g->getName()) == 0) return g;
	}

	return 0;
}

void		font::cacheGlyph(fontglyph *g)
{
	if(glyphcache == 0)
	{
		glyphcache = (fontglyph**)malloc(sizeof(fontglyph*) * 2);
		glyphcache[0] = g;
		glyphcache[1] = 0;
	}
	else
	{
		int	n;

		for(n = 0;glyphcache[n] != 0;n++);

		glyphcache = (fontglyph**)realloc(glyphcache,sizeof(fontglyph*) * (n + 2));
		glyphcache[n] = g;
		glyphcache[n + 1] = 0;
	}
}

void		font::setFileName(char *n)
{
	if(filename) free(filename);
	filename = strdup(n);
	initialize();
}

char		*font::getFileName()
{
	return filename;
}

	
